"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RenderGRPCPluginV2 = void 0;
const grpc = require("@grpc/grpc-js");
const protoLoader = require("@grpc/proto-loader");
const promClient = require("prom-client");
const security_1 = require("../../config/security");
const browser_1 = require("../../browser");
const types_1 = require("./types");
const Sanitizer_1 = require("../../sanitizer/Sanitizer");
const constants_1 = require("@grpc/grpc-js/build/src/constants");
const rendererV2PackageDef = protoLoader.loadSync(__dirname + '/../../../proto/rendererv2.proto', {
    keepCase: true,
    longs: String,
    enums: String,
    defaults: true,
    oneofs: true,
});
const pluginV2PackageDef = protoLoader.loadSync(__dirname + '/../../../proto/pluginv2.proto', {
    keepCase: true,
    longs: String,
    enums: String,
    defaults: true,
    oneofs: true,
});
const sanitizerPackageDef = protoLoader.loadSync(__dirname + '/../../../proto/sanitizer.proto', {
    keepCase: true,
    longs: String,
    enums: String,
    defaults: true,
    oneofs: true,
});
const rendererV2ProtoDescriptor = grpc.loadPackageDefinition(rendererV2PackageDef);
const pluginV2ProtoDescriptor = grpc.loadPackageDefinition(pluginV2PackageDef);
const sanitizerProtoDescriptor = grpc.loadPackageDefinition(sanitizerPackageDef);
class RenderGRPCPluginV2 {
    constructor(config, log) {
        this.config = config;
        this.log = log;
    }
    async grpcServer(server) {
        const metrics = setupMetrics();
        const browser = (0, browser_1.createBrowser)(this.config.rendering, this.log, metrics);
        const pluginService = new PluginGRPCServer(browser, this.log, (0, Sanitizer_1.createSanitizer)(), this.config.plugin.security);
        const rendererServiceDef = rendererV2ProtoDescriptor['pluginextensionv2']['Renderer']['service'];
        server.addService(rendererServiceDef, pluginService);
        const pluginServiceDef = pluginV2ProtoDescriptor['pluginv2']['Diagnostics']['service'];
        server.addService(pluginServiceDef, pluginService);
        const sanitizerServiceDef = sanitizerProtoDescriptor['pluginextensionv2']['Sanitizer']['service'];
        server.addService(sanitizerServiceDef, pluginService);
        metrics.up.set(1);
        let browserVersion = 'unknown';
        let labelValue = 1;
        try {
            browserVersion = await browser.getBrowserVersion();
        }
        catch (err) {
            this.log.error('Failed to get browser version', 'err', err);
            labelValue = 0;
        }
        metrics.browserInfo.labels(browserVersion).set(labelValue);
        if (browserVersion !== 'unknown') {
            this.log.debug('Using browser version', 'browserVersion', browserVersion);
        }
        await pluginService.start(browserVersion);
    }
}
exports.RenderGRPCPluginV2 = RenderGRPCPluginV2;
class PluginGRPCServer {
    constructor(browser, log, sanitizer, securityCfg) {
        this.browser = browser;
        this.log = log;
        this.sanitizer = sanitizer;
        this.securityCfg = securityCfg;
    }
    async start(browserVersion) {
        this.browserVersion = browserVersion;
        await this.browser.start();
    }
    async render(call, callback) {
        const abortController = new AbortController();
        const { signal } = abortController;
        const req = call.request;
        const headers = {};
        if (!req) {
            return callback({ code: constants_1.Status.INVALID_ARGUMENT, details: 'Request cannot be null' });
        }
        if (!(0, security_1.isAuthTokenValid)(this.securityCfg, req.authToken)) {
            return callback({ code: constants_1.Status.UNAUTHENTICATED, details: 'Unauthorized request' });
        }
        if (req.url && !(req.url.startsWith('http://') || req.url.startsWith('https://'))) {
            return callback({ code: constants_1.Status.INVALID_ARGUMENT, details: 'Forbidden query url protocol' });
        }
        if (req.headers) {
            for (const key in req.headers) {
                if (req.headers.hasOwnProperty(key)) {
                    const h = req.headers[key];
                    headers[key] = h.values.join(';');
                }
            }
        }
        const options = {
            url: req.url,
            width: req.width,
            height: req.height,
            filePath: req.filePath,
            timeout: req.timeout,
            renderKey: req.renderKey,
            domain: req.domain,
            timezone: req.timezone,
            deviceScaleFactor: req.deviceScaleFactor,
            headers: headers,
            encoding: req.encoding,
        };
        this.log.debug('Render request received', 'url', options.url);
        call.on('cancelled', (err) => {
            this.log.debug('Connection closed', 'url', options.url, 'error', err);
            abortController.abort();
        });
        let errStr = '';
        try {
            await this.browser.render(options, signal);
        }
        catch (err) {
            this.log.error('Render request failed', 'url', options.url, 'error', err.toString());
            errStr = err.toString();
        }
        callback(null, { error: errStr });
    }
    async renderCsv(call, callback) {
        const abortController = new AbortController();
        const { signal } = abortController;
        const req = call.request;
        const headers = {};
        if (!req) {
            return callback({ code: constants_1.Status.INVALID_ARGUMENT, details: 'Request cannot be null' });
        }
        if (!(0, security_1.isAuthTokenValid)(this.securityCfg, req.authToken)) {
            return callback({ code: constants_1.Status.UNAUTHENTICATED, details: 'Unauthorized request' });
        }
        if (req.url && !(req.url.startsWith('http://') || req.url.startsWith('https://'))) {
            return callback({ code: constants_1.Status.INVALID_ARGUMENT, details: 'Forbidden query url protocol' });
        }
        if (req.headers) {
            for (const key in req.headers) {
                if (req.headers.hasOwnProperty(key)) {
                    const h = req.headers[key];
                    headers[key] = h.values.join(';');
                }
            }
        }
        const options = {
            url: req.url,
            filePath: req.filePath,
            timeout: req.timeout,
            renderKey: req.renderKey,
            domain: req.domain,
            timezone: req.timezone,
            headers: headers,
        };
        this.log.debug('Render request received', 'url', options.url);
        call.on('cancelled', (err) => {
            this.log.debug('Connection closed', 'url', options.url, 'error', err);
            abortController.abort();
        });
        let errStr = '';
        let fileName = '';
        try {
            const result = await this.browser.renderCSV(options, signal);
            fileName = result.fileName || '';
        }
        catch (err) {
            this.log.error('Render request failed', 'url', options.url, 'error', err.toString());
            errStr = err.toString();
        }
        callback(null, { error: errStr, fileName });
    }
    async checkHealth(_, callback) {
        const jsonDetails = Buffer.from(JSON.stringify({
            browserVersion: this.browserVersion,
        }));
        callback(null, { status: types_1.HealthStatus.OK, message: 'Success', jsonDetails: jsonDetails });
    }
    async collectMetrics(_, callback) {
        const payload = Buffer.from(await promClient.register.metrics());
        callback(null, { metrics: { prometheus: payload } });
    }
    async sanitize(call, callback) {
        const grpcReq = call.request;
        if (!(0, security_1.isAuthTokenValid)(this.securityCfg, grpcReq.authToken)) {
            return callback({ code: constants_1.Status.UNAUTHENTICATED, details: 'Unauthorized request' });
        }
        const req = {
            content: grpcReq.content,
            config: JSON.parse(grpcReq.config.toString()),
            configType: grpcReq.configType,
        };
        this.log.debug('Sanitize request received', 'contentLength', req.content.length, 'name', grpcReq.filename);
        try {
            const sanitizeResponse = this.sanitizer.sanitize(req);
            callback(null, { error: '', sanitized: sanitizeResponse.sanitized });
        }
        catch (e) {
            this.log.error('Sanitization failed', 'contentLength', req.content.length, 'name', grpcReq.filename, 'error', e.stack);
            callback(null, { error: e.stack, sanitized: Buffer.from('', 'binary') });
        }
    }
}
const setupMetrics = () => {
    promClient.collectDefaultMetrics();
    return {
        up: new promClient.Gauge({
            name: 'up',
            help: '1 = up, 0 = not up',
        }),
        browserInfo: new promClient.Gauge({
            name: 'grafana_image_renderer_browser_info',
            help: "A metric with a constant '1 value labeled by version of the browser in use",
            labelNames: ['version'],
        }),
        durationHistogram: new promClient.Histogram({
            name: 'grafana_image_renderer_step_duration_seconds',
            help: 'duration histogram of browser steps for rendering an image labeled with: step',
            labelNames: ['step'],
            buckets: [0.1, 0.3, 0.5, 1, 3, 5, 10, 20, 30],
        }),
    };
};
//# sourceMappingURL=grpc_plugin.js.map